//
//  dialogs.cpp
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include "dialogs.h"
#include "icons.h"
#include "utils.h"
#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>
#include <glibmm/miscutils.h>
#include <libgnomevfsmm/init.h>                                                         

#if HAVE_EXIF
# include "exif_interface.h"
#endif

using namespace Gx::Dialog;

Header::Header(const Glib::ustring& text,
	       const Gx::Image& img)
{
	char* markup = g_markup_printf_escaped("<big><b>%s</b></big>", text.data());
	header.set_markup(markup);

	markup = g_markup_printf_escaped("<small>%s</small>", img.get_name().data());
	name.set_markup(markup);

	Image::generate_thumbnail(img, &thumb, 48);

	header.set_alignment(Gtk::ALIGN_LEFT);
	name.set_alignment(Gtk::ALIGN_LEFT);
	thumb.set_alignment(0.95);

	// packing:
	vbox.pack_start(header, false, false, 0);
	vbox.pack_start(name, false, false, 0);

	hbox.pack_start(vbox, false, false, 0);
	hbox.pack_start(thumb, true, true, 0);

	pack_start(hbox);
	pack_start(sep);
}
	       

Info::Info(const Gx::Image& img)
	: header("Image info", img),
	  table(6, 2, false),
	  close(Gtk::Stock::CLOSE)
{
	// Create labels:
	label["path"] = manage(new Gtk::Label("Stored in:"));
	label["mime"] = manage(new Gtk::Label("Mime type:"));
	label["size"] = manage(new Gtk::Label("Filesize:"));
	label["width"] = manage(new Gtk::Label("Width:"));
	label["height"] = manage(new Gtk::Label("Height:"));
	label["alpha"] = manage(new Gtk::Label("Alpha channel:"));

	// Create entries:
	entry["path"] = manage(new Gtk::Entry());
	entry["mime"] = manage(new Gtk::Entry());
	entry["size"] = manage(new Gtk::Entry());
	entry["width"] = manage(new Gtk::Entry());
	entry["height"] = manage(new Gtk::Entry());
	entry["alpha"] = manage(new Gtk::Entry());

	// dialog settings:
	set_icon(Gdk::Pixbuf::create_from_inline(-1, icon, false));
	set_title("Image info");
	set_position(Gtk::WIN_POS_CENTER_ALWAYS);

	add_action_widget(close, Gtk::RESPONSE_NONE);

	/***
        	FIRST PAGE -- GENERAL INFO
		                             ***/

	// load informations and store them:
	std::string path = img.get_path();
	Glib::RefPtr<Gnome::Vfs::FileInfo> info = Utils::get_file_info(path);

	entry["path"]->set_text(img.get_dirname());
	entry["mime"]->set_text(info->get_mime_type());
	
	unsigned buf_len = 24;
	char* buf = new char[buf_len];

	g_snprintf(buf, buf_len, "%d KB", Utils::umanize_size(info->get_size()));
	entry["size"]->set_text(buf);

	g_snprintf(buf, buf_len, "%d pixels", img.get_width());
	entry["width"]->set_text(buf);

	g_snprintf(buf, buf_len, "%d pixels", img.get_height());
	entry["height"]->set_text(buf);

	delete[] buf;

	std::string alpha = img._get_pixbuf()->get_has_alpha() ? "Yes" : "No";
	entry["alpha"]->set_text(alpha);

	// labels settings:
	Gtk::AlignmentEnum align = Gtk::ALIGN_LEFT;

	label["path"]->set_alignment(align);
	label["mime"]->set_alignment(align);
	label["size"]->set_alignment(align);
	label["width"]->set_alignment(align);
	label["height"]->set_alignment(align);
	label["alpha"]->set_alignment(align);

	// entry settings:
	entry["path"]->set_has_frame(false);
	entry["mime"]->set_has_frame(false);
	entry["size"]->set_has_frame(false);
	entry["width"]->set_has_frame(false);
	entry["height"]->set_has_frame(false);
	entry["alpha"]->set_has_frame(false);

	entry["path"]->set_editable(false);
	entry["mime"]->set_editable(false);
	entry["size"]->set_editable(false);
	entry["width"]->set_editable(false);
	entry["height"]->set_editable(false);
	entry["alpha"]->set_editable(false);

	int width_chars = 32;

	entry["path"]->set_width_chars(width_chars);
	entry["mime"]->set_width_chars(width_chars);
	entry["size"]->set_width_chars(width_chars);
	entry["width"]->set_width_chars(width_chars);
	entry["height"]->set_width_chars(width_chars);
	entry["alpha"]->set_width_chars(width_chars);

	// table settings:
	table.set_col_spacings(10);
	table.set_row_spacings(5);
	table.attach(*label["path"], 0, 1, 1, 2);
	table.attach(*label["mime"], 0, 1, 2, 3);
	table.attach(*label["size"], 0, 1, 3, 4);
	table.attach(*label["width"], 0, 1, 4, 5);
	table.attach(*label["height"], 0, 1, 5, 6);
	table.attach(*label["alpha"], 0, 1, 6, 7);

	table.attach(*entry["path"], 1, 2, 1, 2);
	table.attach(*entry["mime"], 1, 2, 2, 3);
	table.attach(*entry["size"], 1, 2, 3, 4);
	table.attach(*entry["width"], 1, 2, 4, 5);
	table.attach(*entry["height"], 1, 2, 5, 6);
	table.attach(*entry["alpha"], 1, 2, 6, 7);

	// packing:
	nb.append_page(table, "General Info");

	/***
        	SECOND PAGE -- EXIF INFO
		                             ***/
#if HAVE_EXIF
	Exif exif_info(path);

#else
//	buffer = Gtk::TextBuffer::create();
//	textv.set_buffer(buffer);

//	buffer->set_text(""
//			 "gx hasn't been compiled with libexif support.\n"
//			 "So, to use this feature, you have to recompile gx.\n"
//			 "Otherwise, you can install gx binary package.\n"
//			 "Download it from http://perticone.homelinux.net/c++/gx/");

//	vbox0.pack_start(textv);

#endif // HAVE_EXIF

//	nb.append_page(vbox0, "Exif Info");
//	nb.set_scrollable();
	
	// end packing:
	Gtk::VBox* dialog_vbox = get_vbox();
	dialog_vbox->pack_start(header);
	dialog_vbox->pack_start(nb);

	show_all_children();
}

Saturate::Saturate(const Gx::Image& img)
	: header("Saturate", img),
	  adj(1.0, -10.0, 12.0, 0.5, 5),
	  scale(adj),
	  apply(Gtk::Stock::APPLY),
	  cancel(Gtk::Stock::CANCEL)
{
	// dialog settings:
	add_action_widget(apply, Gtk::RESPONSE_APPLY);
	add_action_widget(cancel, Gtk::RESPONSE_CANCEL);

	apply.set_sensitive(false);

	set_title("Saturate");
	set_border_width(10);

	// Pointer to Gx::Image:
	pImage = &img;

	// Setting reset button:
	Gtk::Image* im_reset = manage(new Gtk::Image(Gtk::Stock::UNDO, Gtk::ICON_SIZE_BUTTON));
	reset.set_image(*im_reset);
	reset.set_label("Reset");

	// Backup pixbuf:
	ref_img = img._get_pixbuf()->copy();
	preview_pixbuf = ref_img->copy();

	fit_preview();	

	// Signals:
	adj.signal_value_changed().connect(sigc::mem_fun(*this,
							 &Saturate::update_preview));
	reset.signal_clicked().connect(sigc::mem_fun(*this,
						     &Saturate::on_reset));
	
	// Packing:
	hbox.pack_end(reset, false, false, 0);

	Gtk::VBox* vbox = get_vbox();
	vbox->pack_start(header);
	vbox->pack_start(image);
	vbox->pack_start(hbox, false, false, 0);
	vbox->pack_start(scale, false, false, 0);

	show_all_children();
}

void Saturate::fit_preview()
{
	const unsigned img_x = pImage->get_width();
	const unsigned img_y = pImage->get_height();

	unsigned desired_x = 512;
	unsigned desired_y = 512;
	
	if(img_x > desired_x or img_y > desired_y) {
		unsigned zoomed_img_x = desired_x;
		unsigned zoomed_img_y = desired_x * img_y / img_x;
				
		if(zoomed_img_y > desired_y) {
			zoomed_img_x = desired_y * img_x / img_y;
			zoomed_img_y = desired_y;
		}
				
		unsigned zoom_factor = 100 * zoomed_img_x / img_x;
				
		preview_pixbuf = preview_pixbuf->scale_simple(img_x * zoom_factor / 100,
							      img_y * zoom_factor / 100,
							      Gdk::INTERP_TILES);
	}
			
	image.set(preview_pixbuf);
}

void Saturate::update_preview()
{
	if(not apply.is_sensitive())
		apply.set_sensitive();

	Glib::RefPtr<Gdk::Pixbuf> tmp_pixbuf = preview_pixbuf->copy();
	tmp_pixbuf->saturate_and_pixelate(tmp_pixbuf, adj.get_value(),
					  false);

	image.set(tmp_pixbuf);
}

Scale::Scale(Gx::Image& img)
	: header("Scale image", img),
	  img_width(img.get_width()),
	  img_height(img.get_height()),
	  ok(Gtk::Stock::OK),
	  cancel(Gtk::Stock::CANCEL),
	  block("_Block ratio", true),
	  l_header_sz("", 0.02, 0.50),
	  l_width("Width:"),
	  l_height("Height:"),
	  l_header_qual("", 0.02, 0.50),
	  l_interp("Interpolation:"),
	  adj_width(img_width, 1, img_width * 4, 1, 10, 1),
	  adj_height(img_height, 1, img_height * 4, 1, 10, 1),
	  s_width(adj_width, 0, 0),
	  s_height(adj_height, 0, 0),
	  hbox0(true, 0),
	  hbox1(true, 0)
{
	set_icon(Gdk::Pixbuf::create_from_inline(-1, icon, false));
	set_title("Scale image");
	set_default_size(280, 320);
	set_position(Gtk::WIN_POS_CENTER_ALWAYS);

	add_action_widget(ok, Gtk::RESPONSE_OK);
	add_action_widget(cancel, Gtk::RESPONSE_CANCEL);

	// setting reset button:
	Gtk::Image* im_reset = manage(new Gtk::Image(Gtk::Stock::UNDO, Gtk::ICON_SIZE_BUTTON));
	reset.set_image(*im_reset);
	reset.set_label("Reset");

	// setting labels:
	char* markup;
	markup = g_markup_printf_escaped("<b>%s</b>", "Image size:");
	l_header_sz.set_markup(markup);

	markup = g_markup_printf_escaped("<b>%s</b>", "Quality:");
	l_header_qual.set_markup(markup);

	g_free(markup);

	update_l_size(img_width, img_height);

	// setting treemodel and combo box:
	/// create the model:
	ref_model = Gtk::ListStore::create(columns);
	combo.set_model(ref_model);
	
	/// fill the combo box's model:
	Gtk::TreeModel::Row row = *(ref_model->append());
	row[columns.interp] = "Nearest";

	row = *(ref_model->append());
	row[columns.interp] = "Tiles";

	row = *(ref_model->append());
	row[columns.interp] = "Bilinear";

	row = *(ref_model->append());
	row[columns.interp] = "Hyper";

	/// Add the model column to the combo box:
	combo.pack_start(columns.interp);
	combo.set_active(2); // Bilinear

	// misc settings:
	block.set_active();
	reset.set_sensitive(false);
	ok.set_sensitive(false);

	// signals:
	s_width.signal_value_changed().connect(sigc::mem_fun(*this, 
							     &Scale::on_width_changed));
	s_height.signal_value_changed().connect(sigc::mem_fun(*this, 
							      &Scale::on_height_changed));

	reset.signal_clicked().connect(sigc::mem_fun(*this, &Scale::on_reset));

	ok.signal_clicked().connect(sigc::bind(mem_fun(*this, &Scale::on_scale),
					       &img));

	// packing:
	hbox0.pack_start(l_width, false, true);
	hbox0.pack_start(s_width, false, false);

	hbox1.pack_start(l_height, false, true);
	hbox1.pack_start(s_height, false, false);

	hbox2.pack_start(l_interp);
	hbox2.pack_start(combo);

	hbox_but.pack_end(reset, false, false, 0);

	// end packing:
	Gtk::VBox* main_vbox = get_vbox();
	main_vbox->pack_start(header);
	main_vbox->pack_start(l_header_sz);
	main_vbox->pack_start(hbox0);
	main_vbox->pack_start(hbox1);
	main_vbox->pack_start(l_size);
	main_vbox->pack_start(block);
	main_vbox->pack_start(hbox_but);
	main_vbox->pack_start(l_header_qual);
	main_vbox->pack_start(hbox2);

	show_all_children();
}

void Scale::on_reset()
{
	s_width.set_value(img_width);
	s_height.set_value(img_height);

	update_l_size(img_width, img_height);

	if(not block.get_active())
		block.set_active();
	
	if(ok.is_sensitive())
		ok.set_sensitive(false);

	reset.set_sensitive(false);
}

void Scale::on_width_changed()
{
	if(block.get_active()) {
		int height_value = static_cast<int>(img_height * s_width.get_value() / img_width);
		float diff = height_value - s_height.get_value();

		if(diff > 1 or diff < -1)
			s_height.set_value(height_value);
	}

	update_l_size(static_cast<int>(s_width.get_value()), 
		      static_cast<int>(s_height.get_value()));

	if(not ok.is_sensitive())
		ok.set_sensitive();

	if(not reset.is_sensitive())
		reset.set_sensitive();
}

void Scale::on_height_changed()
{

	if(block.get_active()) {
		int width_value = static_cast<int>(img_width * s_height.get_value() / img_height);
		float diff = width_value - s_width.get_value();

		if(diff > 1 or diff < -1)
			s_width.set_value(width_value);
	}

	update_l_size(static_cast<int>(s_width.get_value()), 
		      static_cast<int>(s_height.get_value()));

	if(not ok.is_sensitive())
		ok.set_sensitive();

	if(not reset.is_sensitive())
		reset.set_sensitive();
}

void Scale::on_scale(Gx::Image* image) // pointer yes... I know, it's crappy
{
	Gdk::InterpType interp;

	switch(combo.get_active_row_number()) {
	case 0:
		interp = Gdk::INTERP_NEAREST;
		break;

	case 1:
		interp = Gdk::INTERP_TILES;
		break;

	case 2:
		interp = Gdk::INTERP_BILINEAR;
		break;
		
	case 3:
		interp = Gdk::INTERP_HYPER;
		break;

	default:
		interp = Gdk::INTERP_BILINEAR;
	}

	image->scale(static_cast<int>(s_width.get_value()),
		     static_cast<int>(s_height.get_value()),
		     interp);
}

void Scale::update_l_size(int x, int y)
{
	unsigned buf_len = 8; // 8 bytes should be enough..

	char* str_width = new char[buf_len];
	char* str_height = new char[buf_len];
	sprintf(str_width, "%d", x);
	sprintf(str_height, "%d", y);
	
	Glib::ustring str = Glib::ustring(str_width) + 'x' + Glib::ustring(str_height) + " pixels";
	
	char* markup = g_markup_printf_escaped("<small>%s</small>", str.data());
	l_size.set_markup(markup);

	g_free(markup);

	delete[] str_width;
	delete[] str_height;
}	

Supported::Supported()
	: close(Gtk::Stock::CLOSE)
{
	set_icon(Gdk::Pixbuf::create_from_inline(-1, icon, false));
	set_title("Supported formats");
	set_default_size(0, 385);

	set_position(Gtk::WIN_POS_CENTER_ALWAYS);

	add_action_widget(close, Gtk::RESPONSE_NONE);

	// Create the treemodel:
	refModel = Gtk::ListStore::create(columns);
	treeview.set_model(refModel);

	// Fill the treeview's model:
	Gtk::TreeModel::Row row;

	FormatList l = Gdk::Pixbuf::get_formats();

	for(FormatCIter i=l.begin(); i!=l.end(); i++) {
		row = *(refModel->append());

		row[columns.descr] = (*i).get_description();
		row[columns.name] = (*i).get_name();
		
		if((*i).is_writable()) {
			row[columns.writable] = Gtk::Widget::render_icon(Gtk::Stock::YES,
									 Gtk::ICON_SIZE_MENU,
									 "gtk-yes");
			row[columns.not_writable] = false;
		}
		else {
			row[columns.writable] = Gtk::Widget::render_icon(Gtk::Stock::NO,
									 Gtk::ICON_SIZE_MENU,
									 "gtk-no");
			row[columns.not_writable] = true;
		}
	}
	
	// Treeview options:
	treeview.set_rules_hint();

	// Append the columns:
	treeview.append_column("Type", columns.descr);
	treeview.append_column("Extension", columns.name);
	treeview.append_column("Able to save", columns.writable);
	
	// Sort options:
	{
		Gtk::TreeView::Column* pColumn = treeview.get_column(0);
		pColumn->set_sort_column(columns.descr);

		pColumn = treeview.get_column(1);
		pColumn->set_sort_column(columns.name);

		pColumn = treeview.get_column(2);
		pColumn->set_sort_column(columns.not_writable);
	}

	// Packing:
	scroll_win.add(treeview);
	scroll_win.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
	
	Gtk::VBox* vbox = get_vbox();
	vbox->pack_start(scroll_win);

	show_all_children();
}
