//
// image.cpp
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include "image.h"
#include "info.h"
#include "utils.h"
#include <gtkmm/adjustment.h>
#include <list>

using namespace Gx;

Image::Image(const std::string& src_path) 
	: interp(Gdk::INTERP_BILINEAR),
	  zoom(100),
	  rotation(0),
	  modified(false),
	  editable(false),
	  writable(false),
	  animated(false),
	  invalid(false),
	  mirrored(false),
	  fitted(true),
	  path(src_path)
{ }

Image::Image(const Image& img)
{ 
	*this = img;
}
       
void Image::save(bool force)
{
	Glib::ustring mime;
	Glib::ustring ext = Utils::extract_ext_from_filename(get_name());

	// adjust fucking extension:
	if(ext == "BMP") ext = "bmp";
	if(ext == "tif" or ext == "TIF" or ext == "TIFF") ext = "tiff";
	if(ext == "JPEG" or ext == "JPG" or ext == "jpg") ext = "jpeg";
	if(ext == "PNG") ext = "png";
	if(ext == "ICO" or ext == "ICON" or ext == "icon") ext = "ico";

	if(force) // ignore mime-type checking... obligatory for "Save as"
		mime = (ext == "jpeg") ? "image/jpeg" : "unknown";

	else { // overwrite an image
		try {
			if(not Utils::is_writable(path))
				throw Error::Unwritable();
		}
		catch(const Gx::Error::Access&) {
			throw;
		}

		mime = Utils::get_mime_type(path);
	}

	// Ok, now we can save it... warning Gdk::Pixbuf::save throws exceptions!
	// However they'll get catched in Gx::Window::save()
	if(mime == "image/jpeg") { // We have to save it with 100% quality:
		UstringList keys;
		UstringList values;
		keys.push_front("quality"); 
		values.push_front("100"); // we should be able to choose for this... blaaaah

		ref_img->save(path, "jpeg",
			      Glib::StringArrayHandle(keys),
			      Glib::StringArrayHandle(values));
	}
	else
			ref_img->save(path, ext);

	modified = not force;
}

// Load an image from pixbuf:
void Image::set_from_pixbuf(Glib::RefPtr<Gdk::Pixbuf> pixbuf) 
{ 
	ref_img = ref_img_zoomed = pixbuf;
	animated = false;
	
	draw(); 
}

// Load an image from pixbuf (animated):
void Image::set_from_pixbuf_anim(Glib::RefPtr<Gdk::PixbufAnimation> pixbuf) 
{ 
	ref_anim = pixbuf;
	ref_img = ref_anim->get_static_image();
	animated = true;
	
	draw(); 
}

// (re)load image from path
void Image::load()
{
	try {
		if(not Utils::is_supported(path)) {
			set_as_invalid();
			return;
		}
	}
	catch(const Gx::Error::Access&) {
		set_as_invalid();
		return;
	}

	invalid = false;
	
	set(path);

	if(get_storage_type() == Gtk::IMAGE_ANIMATION) {
		editable = false;
		animated = true;

		ref_anim = get_animation();
		ref_img = ref_anim->get_static_image();
	}
	else {
		animated = false;
		editable = true;
		ref_img = get_pixbuf();
	}

	if(zoom != 100 and not fitted)
		set_zoom(zoom, true);

	if(modified) {
		try {
			rotate(rotation, true);
		}
		catch(const Error::Rotate& e) {
			g_printerr("gx error: rotation value error (%d) in "
				   "Gx::Image::load()\n"
				   "Please, report this message to %s\n\n", e.get_value(), MAIL);
		}

		if(is_saturated())
			for(SaturationIter i=l_saturation.begin(); i!= l_saturation.end(); i++)
				saturate(*i, true);

		if(mirrored) mirror(true);

		if(is_scaled())
			try {
				if(rotation == -90 or rotation == 90 or rotation == 270)
					for(ScaleIter i=l_scale.begin(); i!=l_scale.end(); i++)
						scale((*i).m_height,
						      (*i).m_width,
						      (*i).m_interp, true);
				else
					for(ScaleIter i=l_scale.begin(); i!=l_scale.end(); i++)
						scale((*i).m_width,
						      (*i).m_height,
						      (*i).m_interp, true);
			}
			catch(const Error::Scale& e) {
				g_printerr("gx error: scale value(s) error in "
					   "Gx::Image::load()\n"
					   "The values are: %d and %d\n"
					   "Please, report this message to %s\n\n", 
					   e.get_width(), e.get_height(), MAIL);
			}
	}
	else
		writable = Utils::is_writable(path);

	draw();
}

void Image::restore()
{
	clear();

	interp = Gdk::INTERP_TILES;
	zoom = 100;
	rotation = 0;
	modified = false;
	mirrored = false;
	fitted = true;

	while(not s_undo.empty())
		s_undo.pop();

//	while(not s_redo.empty())
//		s_redo.pop();

	if(not l_saturation.empty())
		l_saturation.clear();

	if(not l_scale.empty())
		l_scale.clear();

	load();
}

// Fit the image to the window:
void Image::zoom_fit(int win_x, int win_y, bool force)
{
	int new_zoom = get_fit_zoom_value(win_x, win_y);

	if(not new_zoom)
		return;

	if(new_zoom == 100) {
		reset_zoom();
		fitted = true;
		return;
	}

	if(new_zoom != zoom or force) {
		ref_img_zoomed = ref_img->scale_simple(get_width() * new_zoom / 100,
						       get_height() * new_zoom / 100,
						       interp);
		zoom = new_zoom;
		fitted = true;
		draw();
	}
}
	
// Reset zoom to 100%
void Image::reset_zoom()
{
	zoom = 100;
	fitted = false;

	draw();
}

// Handle a zoom of zoom_factor%
void Image::make_zoom_in()
{
	int z = zoom + zoom_factor;

	if(z > max_zoom) 
		z = 400;

	set_zoom(z);
}

// Handle a zoom out of -zoom_factor%
void Image::make_zoom_out()
{
	int x = get_width();
	int y = get_height();

	if(x * zoom - zoom_factor / 100 < 1 or
	   y * zoom - zoom_factor / 100 < 1)
		throw Error::Zoom(zoom);

	set_zoom(zoom - zoom_factor);
}

void Image::saturate(double value, bool update)
{
	if(not update) {
		l_saturation.push_back(value);
		s_undo.push(std::make_pair(ref_img->copy(), ACTION_SATURATE));

//		while(not s_redo.empty())
//			s_redo.pop();
	}

	ref_img->saturate_and_pixelate(ref_img, value, false);
	ref_img_zoomed->saturate_and_pixelate(ref_img_zoomed, value, false);

	if(not update) {
		modified = true;
		if(zoom != 0)
			set_zoom(zoom, true);

		draw();
	}
}

// Rescale the image:
void Image::scale(int width, int height, Gdk::InterpType interp, bool update)
{
	if(width < 1 or height < 1)
		throw Error::Scale(width, height);

	if(not update) {
		if(rotation == -90 or rotation == 90)
			l_scale.push_back(ScaleData(height, width, interp));
		else
			l_scale.push_back(ScaleData(width, height, interp));		

		s_undo.push(std::make_pair(ref_img->copy(), ACTION_SCALE));

//		while(not s_redo.empty())
//			s_redo.pop();
	}

	ref_img_zoomed = ref_img = ref_img->scale_simple(width, height, interp);

	if(not fitted)
		set_zoom(zoom, true);

	if(not update) {
		modified = true;
		draw();
	}
}

void Image::mirror(bool update)
{
	if(not update) {
		s_undo.push(std::make_pair(ref_img->copy(), ACTION_MIRROR));

//		while(not s_redo.empty())
//			s_redo.pop();

		mirrored = not mirrored;

		modified = true;
	}

	ref_img = ref_img->flip();
	ref_img_zoomed = ref_img_zoomed->flip();

	if(not update) draw();
}

// It applies a rotation of 'angle' degrees to the image.
// If 'update' flag is true we have to update the current status.
void Image::rotate(int angle, bool update)
{
	if(not update) {
		Action act = angle > 0 ? ACTION_ROTATE_C : ACTION_ROTATE_CC;
		s_undo.push(std::make_pair(Glib::RefPtr<Gdk::Pixbuf>(), act));

//		while(not s_redo.empty())
//			s_redo.pop();
	}

	if(not angle or angle == 360)
		return;

	switch(angle) {
	case -90: 
	case 270:
		ref_img = ref_img->rotate_simple(Gdk::PIXBUF_ROTATE_COUNTERCLOCKWISE);
		ref_img_zoomed = ref_img_zoomed->rotate_simple(Gdk::PIXBUF_ROTATE_COUNTERCLOCKWISE);
		break;

	case 90: 
		ref_img = ref_img->rotate_simple(Gdk::PIXBUF_ROTATE_CLOCKWISE);
		ref_img_zoomed = ref_img_zoomed->rotate_simple(Gdk::PIXBUF_ROTATE_CLOCKWISE);
		break;

	case -180:
	case 180:
		ref_img = ref_img->rotate_simple(Gdk::PIXBUF_ROTATE_UPSIDEDOWN);
		ref_img_zoomed = ref_img_zoomed->rotate_simple(Gdk::PIXBUF_ROTATE_UPSIDEDOWN);
		break;

	default:
		throw Error::Rotate(angle);
	}

	if(not update) {
		rotation += angle;

		if(rotation >= 360)
			rotation -= 360;

		if(rotation < -90)
			rotation += 360;

		modified = true;
		draw();
	}
	else
		if(rotation != angle)
			rotation = angle;
}

void Image::undo()
{
	if(s_undo.empty())
		return;

//	s_redo.push(std::make_pair(ref_img->copy(), s_undo.top().second));

	Action act = s_undo.top().second;

	if(act != ACTION_ROTATE_C and act != ACTION_ROTATE_CC)
		ref_img_zoomed = ref_img = s_undo.top().first->copy();

	s_undo.pop();

	switch(act) {
	case ACTION_SCALE:
//		l_scale_redo.push_back(l_scale.back());
		l_scale.pop_back();
		break;
		
	case ACTION_SATURATE:
//		l_saturation_redo.push_back(l_saturation.back());
		l_saturation.pop_back();
		break;

	case ACTION_MIRROR:
		mirrored = not mirrored;
		break;

	case ACTION_ROTATE_C:
	{
		int r = rotation;
		rotate(-90, true);

		rotation = r - 90;
		if(rotation < -90) 
			rotation += 360;

		break;
	}
	case ACTION_ROTATE_CC:
	{
		int r = rotation;
		rotate(90, true);

		rotation = r + 90;
		if(rotation >= 360) 
			rotation -= 360;
		break;
	}
	} // switch

	if(not modified) 
		modified = true;

	if(not fitted) 
		set_zoom(zoom, true);

	draw();
}

// void Image::redo()
// {
// 	if(s_redo.empty())
// 		return;

// 	s_undo.push(std::make_pair(ref_img->copy(), s_redo.top().second));

// 	ref_img = ref_img_zoomed = s_redo.top().first->copy();

// 	switch(s_undo.top().second) {
// 	case ACTION_SCALE:
// 		l_scale.push_back(l_scale_redo.back());
// 		l_scale_redo.pop_back();
// 		break;
		
// 	case ACTION_SATURATE:
// 		l_saturation.push_back(l_saturation_redo.back());
// 		l_saturation_redo.pop_back();
// 		break;

// 	case ACTION_MIRROR:
// 		mirrored = not mirrored;
// 		break;

// 	case ACTION_ROTATE:
// 		break;
// 	}

// 	s_redo.pop();
	
// 	modified = true;

// 	if(not fitted)
// 		set_zoom(zoom, true);

// 	draw();
// }

int Image::get_fit_zoom_value(int win_x, int win_y) const
{
	if(not win_x or not win_y)
		return 100;

	int img_x = get_width();
	int img_y = get_height();

	if(img_x <= win_x and img_y <= win_y)
		return 100;

	int zoomed_img_x = win_x;
	int zoomed_img_y = win_x * img_y / img_x;

	if(zoomed_img_y > win_y) {
		zoomed_img_x = win_y * img_x / img_y;
		zoomed_img_y = win_y;
	}

	return 100 * zoomed_img_x / img_x;
}

// Actually *this* performs zooms 
void Image::set_zoom(int z, bool update)
{

	if(not update and zoom == z)
		return;

	if(z > max_zoom)
		z = max_zoom;

	if(z < 0)
		return;

	int x = get_width();
	int y = get_height();

	int new_x = x * z / 100;
	int new_y = y * z / 100;

	ref_img_zoomed = ref_img->scale_simple(new_x, new_y, interp);

// 	ref_img_zoomed = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, ref_img->get_has_alpha(), 
// 					     ref_img->get_bits_per_sample(),
// 					     new_x, new_y);
	
// 	ref_img->composite_color(ref_img_zoomed,
// 				 0, 0,
// 				 new_x, new_y,
// 				 0, 0,
// 				 static_cast<double>(z) / 100,
// 				 static_cast<double>(z) / 100,
// 				 interp,
// 				 255,
// 				 x, y,
// 				 16,
// 				 0x00999999,
// 				 0x00666666);

	if(not update) {
		fitted = false;
		zoom = z;
		draw();
	}
}

void Image::operator=(const Image& a)
{
	s_undo = a.s_undo;
//	s_redo = a.s_redo;
	l_scale = a.l_scale;
	l_saturation = a.l_saturation;

	modified = a.modified;
	editable = a.editable;
	writable = a.writable;
	animated = a.animated;
	invalid = a.invalid;
	mirrored = a.mirrored;
	fitted = a.fitted;
	zoom = a.zoom;
	rotation = a.rotation;
	path = a.path;
}
