// -*- C++ -*-
//
// image.h
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef GX_IMAGE
#define GX_IMAGE

#include <gtkmm/image.h>
#include <glibmm/miscutils.h>
#include <gtkmm/stock.h>
#include <stack>
#include <string>

namespace Gx {

	enum Action {
		ACTION_ROTATE_C,
		ACTION_ROTATE_CC,
		ACTION_SATURATE,
		ACTION_MIRROR,
		ACTION_SCALE
	};

	class Image : public Gtk::Image {

		struct ScaleData {
			ScaleData(int x, int y, Gdk::InterpType i)
				: m_width(x),
				  m_height(y),
				  m_interp(i) { }

			int              m_width;
			int             m_height;
			Gdk::InterpType m_interp;
		};

		typedef std::stack< std::pair<Glib::RefPtr<Gdk::Pixbuf>, Action> > UndoStack;
		typedef std::list<double> SaturationList;
		typedef std::list<ScaleData> ScaleList;

		typedef SaturationList::const_iterator  SaturationIter;
		typedef ScaleList::const_iterator            ScaleIter;
	public:
		// Constructor/Destructor:
		Image(const std::string& path ="");
		Image(const Image&);
		virtual ~Image() { }

		virtual void                  load   ();
		virtual void               cleanup   (); // inline
		virtual void               restore   (); 

		// Inline member functions:
		virtual Glib::RefPtr<Gdk::Pixbuf>                _get_pixbuf  () const;
		virtual Glib::RefPtr<Gdk::PixbufAnimation>   get_anim_pixbuf  () const;

		virtual std::string                        get_path  () const;
		virtual std::string                     get_dirname  () const;
		virtual std::string                        get_name  () const;
		virtual int                               get_width  () const;
		virtual int                              get_height  () const;
		virtual int                         get_actual_zoom  () const;
		virtual int                            get_max_zoom  () const;
		virtual int                         get_zoom_factor  () const;
		virtual int                     get_actual_rotation  () const;
		virtual bool                           is_saturated  () const;
		virtual bool                              is_scaled  () const;
		virtual bool                            is_modified  () const;
		virtual bool                            is_editable  () const;
		virtual bool                            is_writable  () const;
		virtual bool                            is_animated  () const;
		virtual bool                             is_invalid  () const;
		virtual bool                              is_fitted  () const;
		virtual bool                               has_undo  () const;
//		virtual bool                               has_redo  () const;

		// Other public functions:
		virtual void                  save   (bool force =false);
		virtual void       set_from_pixbuf   (const Glib::RefPtr<Gdk::Pixbuf>);
		virtual void  set_from_pixbuf_anim   (const Glib::RefPtr<Gdk::PixbufAnimation>);
		virtual void              set_zoom   (int zoom, bool update =false);
		virtual void              set_path   (const std::string& path); // inline
		virtual void       set_interp_type   (Gdk::InterpType); // inline
		virtual void              zoom_fit   (int x, int y, bool force =false);
		virtual void            reset_zoom   ();
		virtual void          make_zoom_in   ();
		virtual void         make_zoom_out   ();
		virtual void              saturate   (double    value, 
						      bool     update =false);
		virtual void                 scale   (int                 width,
						      int                height,
						      Gdk::InterpType inteptype, 
						      bool update =false);
		virtual void                mirror   (bool update =false);
		virtual void                rotate   (int angle, bool update =false);
		virtual void                  undo   ();
//		virtual void                  redo   ();

		virtual int         get_fit_zoom_value   (int x, int y) const;

		static void  generate_thumbnail(const Gx::Image& img, 
						Gtk::Image* thumb, unsigned size)
		{
			const unsigned img_x = img.get_width();
			const unsigned img_y = img.get_height();

			const unsigned thumb_x = size;
			const unsigned thumb_y = thumb_x;

			Glib::RefPtr<Gdk::Pixbuf> refPixbuf = img._get_pixbuf();
	
			if(img_x > thumb_x or img_y > thumb_y) {
				unsigned zoomed_img_x = thumb_x;
				unsigned zoomed_img_y = thumb_x * img_y / img_x;
				
				if(zoomed_img_y > thumb_y) {
					zoomed_img_x = thumb_y * img_x / img_y;
					zoomed_img_y = thumb_y;
				}
				
				unsigned zoom_factor = 100 * zoomed_img_x / img_x;
				
				refPixbuf = refPixbuf->scale_simple(img_x * zoom_factor / 100,
								    img_y * zoom_factor / 100,
								    Gdk::INTERP_TILES);
			}
			
			thumb->set(refPixbuf);
		}

		// operators:
		virtual void operator=(const Image& a);

		class Error {
		public:
			class Rotate { 
			public:
				Rotate(int r) : _r(r) { }
				int get_value() const { return _r; }

			private:
				int _r;
			};

			class Scale { 
			public:
				Scale(int x, int y) : _x(x), _y(y) { }
				int get_width() const { return _x; }
				int get_height() const { return _y; }

			private:
				int _x, _y;
			};

			class Unwritable { };

			class Zoom { 
			public:
				Zoom(unsigned z) : _z(z) { }
				unsigned get_value() const { return _z; }

			private:
				unsigned _z;
			};
		};

	protected:
		virtual void            draw   (); // inline
		virtual void  set_as_invalid   (); // inline

	private:
		Glib::RefPtr<Gdk::Pixbuf>          ref_img_zoomed;
		Glib::RefPtr<Gdk::Pixbuf>                 ref_img;
		Glib::RefPtr<Gdk::PixbufAnimation>       ref_anim;

		// undo/redo stuffs:
		UndoStack                     s_undo;
//		UndoStack                     s_redo;

		// utilities:
		SaturationList       l_saturation;
		ScaleList                 l_scale;
//		SaturationList  l_saturation_redo;
//		ScaleList            l_scale_redo;


		// Properties:
		bool            modified;
		bool            editable;
		bool            writable;
		bool            animated;
		bool             invalid;
		bool            mirrored;
		bool              fitted;
		int                 zoom;
		int             rotation;
		std::string         path;

		Gdk::InterpType           interp; // default interp

		// Static values:
		static const int zoom_factor = 20; // %
		static const int max_zoom = 400 + zoom_factor; // %
	};

	// Inline implementations:
	inline Glib::RefPtr<Gdk::Pixbuf> Image::_get_pixbuf() const
	{	
		return ref_img;
	}

	inline Glib::RefPtr<Gdk::PixbufAnimation> Image::get_anim_pixbuf() const
	{
		return ref_anim;
	}

	inline std::string Image::get_path() const
	{
		return path;
	}

	inline std::string Image::get_dirname() const
	{

		return Glib::path_get_dirname(path);
	}

	inline std::string Image::get_name() const
	{
		return Glib::path_get_basename(path);
	}

	inline int Image::get_width() const
	{
		if(ref_img)
			return ref_img->get_width();
		else
			return 0;
	}

	inline int Image::get_height() const
	{
		if(ref_img)
			return ref_img->get_height();
		else
			return 0;
	}

	inline int Image::get_actual_zoom() const
	{
		return zoom;
	}

	inline int Image::get_max_zoom() const
	{
		return max_zoom;
	}

	inline int Image::get_zoom_factor() const
	{
		return zoom_factor;
	}

	inline int Image::get_actual_rotation() const
	{
		return rotation;
	}

	inline bool Image::is_saturated() const
	{
		return not l_saturation.empty();
	}

	inline bool Image::is_scaled() const
	{
		return not l_scale.empty();
	}

	inline bool Image::is_modified() const
	{
		return modified;
	}

	inline bool Image::is_editable() const
	{
		return editable;
	}

	inline bool Image::is_writable() const
	{
		return writable;
	}

	inline bool Image::is_animated() const
	{
		return animated;
	}

	inline bool Image::is_invalid() const
	{
		return invalid;
	}

	inline bool Image::is_fitted() const
	{
		return fitted;
	}

	inline bool Image::has_undo() const
	{
		return not s_undo.empty();
	}

//	inline bool Image::has_redo() const
//	{
//		return not s_redo.empty();
//	}

	inline void Image::set_path(const std::string& new_path)
	{
		path = new_path;
	}

	inline void Image::set_interp_type(Gdk::InterpType type)
	{
		interp = type;
	}

	inline void Image::cleanup()
	{
		if(invalid)
			return;

		if(animated)
			ref_anim.clear();
		else {
			ref_img.clear();
			get_pixbuf().clear();
		}

		clear();
	}

	inline void Image::draw()
	{
		if(invalid)
			set(Gtk::Stock::MISSING_IMAGE, Gtk::ICON_SIZE_LARGE_TOOLBAR);
		else if(animated)
			set(ref_anim);
		else if(zoom == 100)
			set(ref_img);
		else
			set(ref_img_zoomed);
	}

	inline void Image::set_as_invalid()
	{
		invalid = true;
		editable = animated = false;

		draw();
	}

	
} // Gx

#endif // GX_IMAGE
