//
//  utils.cpp
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include "image.h"
#include "utils.h"
#include "formats.h"
#include <libgnomevfsmm/directory-handle.h>
#include <libgnomevfsmm/handle.h>

using namespace Gx;
using namespace Gnome;

namespace Gx { namespace Utils {

	Formats formats_object;

} } // Gx::Utils

UstringList Utils::get_supported_types()
{
	return formats_object.get_supported_formats();
}

UstringList Utils::get_writable_types()
{
	return formats_object.get_supported_formats();
}

// Parse Glib::FileError codes
Glib::ustring Utils::parse_io_error(Glib::FileError::Code code)
{
	Glib::ustring msg = "Error: ";

	switch(code) {
	case Glib::FileError::NAME_TOO_LONG:
		msg += "Name is too long.";
		break;
				
	case Glib::FileError::READONLY_FILESYSTEM:
		msg += "File system is in read-only mode.";
		break;

	case Glib::FileError::NO_SPACE_LEFT:
		msg += "No space left on device.";
		break;

	case Glib::FileError::NOT_ENOUGH_MEMORY:
		msg += "No enough memory.";
		break;
				
	case Glib::FileError::TOO_MANY_OPEN_FILES:
		msg += "Too many open files.";
		break;

	case Glib::FileError::IO_ERROR:
		msg += "Input/Output error.";
		break;
		
	default:
		msg += "Permission denied.";
	}

	return msg;
}

// Parse Gdk::PixbufError codes
Glib::ustring Utils::parse_pixbuf_error(Gdk::PixbufError::Code code)
{
	Glib::ustring msg = "Error: ";

	switch(code) {
	case Gdk::PixbufError::CORRUPT_IMAGE:
		msg += "Image corrupted.";
		break;

	case Gdk::PixbufError::INSUFFICIENT_MEMORY:
		msg += "Insufficient memory.";
		break;

	case Gdk::PixbufError::UNKNOWN_TYPE:
		msg += "Unsupported type.";
		break;

	case Gdk::PixbufError::UNSUPPORTED_OPERATION:
		msg += "Unsupported operation.";
		break;

	default:
		msg += "I cannot save the image.";
	}

	return msg;
}

// Generate a list of images based on arguments passed to gx
FileList Utils::generate_image_list(int argc, char** argv)
{
	FileList l;

	for(int i=1; i<argc; i++) {
		std::string dirname = Glib::path_get_dirname(argv[i]);
		std::string path;

		if(dirname == ".")
			path = Glib::build_filename(Glib::get_current_dir(),
						    Glib::path_get_basename(argv[i]));
		else if(dirname.substr(0, 2) == "..")
			path = Glib::build_filename(Glib::get_current_dir(),
						    argv[i]);
		else
			path = argv[i];

		l.push_back(path);
	}

	return l;
}

// Visit a directory and returns a list of images contained in it
Gx::FileList Utils::visit_directory(const std::string& path)
{
	Vfs::DirectoryHandle   dir;
	FileList              list;
	std::string           name;

	try {
		dir.open(path);
	}
	catch(const Vfs::exception& excp) {
		throw Error::Dir(path);
	}

	bool has_next = true;
	while(has_next) {
		std::string name = dir.read_next(has_next)->get_name();
		std::string complete_name = Glib::build_filename(path, name);

		if(not name.size() or name.at(0) == '.') // ignores hidden files
			continue;
		try {
			if(is_supported(complete_name))
				list.push_back(complete_name);
		}
		catch(const Error::Access&) {
			continue;
		}
	}

	dir.close();

	return list;
}

Glib::RefPtr<Vfs::FileInfo> Utils::get_file_info(const std::string& path)
{
	Glib::RefPtr<Vfs::FileInfo> info;

	try {
		Glib::RefPtr<Vfs::Uri> uri = Vfs::Uri::create(path);
		info = uri->get_file_info(Vfs::FILE_INFO_GET_MIME_TYPE);
	}
	catch(const Vfs::exception excp) {
		throw Error::Access(path);
	}

	return info;
}

Glib::ustring Utils::get_mime_type(const std::string& path)
{
	Glib::ustring mime;

	try {
		Glib::RefPtr<Vfs::FileInfo> info = get_file_info(path);
		mime = info->get_mime_type();
	}
	catch(const Vfs::exception excp) {
		throw Error::Access(path);
	}

	return mime;
}

bool Utils::format_supported(const std::string& mime)
{
        UstringList l = formats_object.get_supported_formats();

        for(UstringCIter i=l.begin(); i!=l.end(); i++)
                if(mime == *i)
                        return true;

        return false;
}

bool Utils::format_writable(const std::string& mime)
{
	UstringList l = formats_object.get_writable_formats();

        for(UstringCIter i=l.begin(); i!=l.end(); i++)
                if(mime == *i)
                        return true;

        return false;
}
