// -*- C++ -*-
//
// utils.h
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef GX_UTILS
#define GX_UTILS

#include <gdkmm/pixbuf.h>
#include <glibmm/ustring.h>
#include <glibmm/fileutils.h>
#include <libgnomevfsmm/uri.h>
#include <sys/stat.h>
#include <unistd.h>
#include <list>
#include <string>

namespace Gx {

	typedef std::list<std::string>          FileList;
	typedef FileList::const_iterator       FileCIter;

	typedef std::list<Glib::ustring>     UstringList;
	typedef UstringList::const_iterator UstringCIter;

	typedef std::list<Gdk::PixbufFormat>  FormatList;
	typedef FormatList::const_iterator   FormatCIter;

	class Error {
	public:
		class Dir { 
		public:
			Dir(const std::string& path) : _path(path) { }
			std::string get_path() const { return _path; }

		private:
			std::string _path;
		};

		class Access { 
		public:
			Access(const std::string& path) : _path(path) { }
			std::string get_path() const { return _path; }

		private:
			std::string _path;
		};

		class Unsupported { 
		public:
			Unsupported(const std::string& path) : _path(path) { }
			std::string get_path() const { return _path; }

		private:
			std::string _path;
		};
	};

} // Gx

namespace Gx { namespace Utils {

	UstringList    get_supported_types  ();	
	UstringList     get_writable_types  ();	

	Glib::ustring       parse_io_error  (Glib::FileError::Code  code);
	Glib::ustring   parse_pixbuf_error  (Gdk::PixbufError::Code code);
	unsigned              umanize_size  (unsigned size_in_bytes); // inline

	FileList      generate_image_list  (int          argc, 
					    char**       argv);

	bool                  is_directory  (const std::string&  path); // inline
	FileList           visit_directory  (const std::string&  path);

	Glib::RefPtr<Gnome::Vfs::FileInfo> get_file_info  (const std::string&  path);
        Glib::ustring                      get_mime_type  (const std::string& path);

	bool     is_supported  (const std::string& path); // inline
	bool format_supported  (const std::string& mime);
	bool      is_writable  (const std::string& path); // inline
	bool  format_writable  (const std::string& mime);

	Glib::ustring extract_ext_from_filename(const std::string& name); // inline

} } // Gx::Utils

inline unsigned Gx::Utils::umanize_size(unsigned sz)
{
	return sz / 1024;
}

inline bool Gx::Utils::is_directory(const std::string& path)
{
	struct stat* info = new struct stat;
	int res = lstat(path.c_str(), info);

	if(res == -1)
		throw Error::Access(path);
       
	mode_t mode = info->st_mode;
	
	return S_ISDIR(mode);
}

inline bool Gx::Utils::is_supported(const std::string& path)
{
	Glib::ustring mime;

	try {
		mime = get_mime_type(path);
	}
	catch(Error::Access&) {
		throw;
	}

	return format_supported(mime);
}

inline bool Gx::Utils::is_writable(const std::string& path)
{
	Glib::ustring mime;

	try {
		mime = get_mime_type(path);
	}
	catch(Error::Access&) {
		throw;
	}

	return format_writable(mime);
}

inline	Glib::ustring Gx::Utils::extract_ext_from_filename(const std::string& name)
{
	size_t i = name.find_last_of('.');
	return name.substr(i+1);
}

#endif // GX_UTILS
