//
// window.cpp
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include "window.h"
#include "info.h"
#include <gtkmm/adjustment.h>
#include <gtkmm/aboutdialog.h>
#include <gtkmm/filechooserdialog.h>
#include <gtkmm/iconfactory.h>
#include <gtkmm/menu.h>
#include <gtkmm/stock.h>
#include <glibmm/fileutils.h>

using namespace Gx;
using namespace Gnome;

Window::Window(FileList& files)
	: first_session(true),
	  main_vbox(false, 0),
	  vbox(false, 0),
	  window_width_setting(0),
	  window_height_setting(0)
{
	// Icon sets:
	icon_set["RotateC"] = Gtk::IconSet(Gdk::Pixbuf::create_from_inline(-1, rotatec, false));
	icon_set["RotateCC"] = Gtk::IconSet(Gdk::Pixbuf::create_from_inline(-1, rotatecc, false));
	icon_set["Mirror"] = Gtk::IconSet(Gdk::Pixbuf::create_from_inline(-1, mirror, false));

	// Setting Stock icons:
	StockID["RotateC"] = Gtk::StockID("gx-rotatec-icon");
	StockID["RotateCC"] = Gtk::StockID("gx-rotatecc-icon");
	StockID["Mirror"] = Gtk::StockID("gx-mirror-icon");

	StockItem["RotateC"] = Gtk::StockItem(StockID["RotateC"], "Rotate Clockwise");
	StockItem["RotateCC"] = Gtk::StockItem(StockID["RotateCC"], "Rotate CounterClockwise");
	StockItem["Mirror"] = Gtk::StockItem(StockID["Mirror"], "Mirror");

	// Stocks settings:
	Glib::RefPtr<Gtk::IconFactory> refFactory = Gtk::IconFactory::create();
	refFactory->add(StockID["RotateC"], icon_set["RotateC"]);
	refFactory->add(StockID["RotateCC"], icon_set["RotateCC"]);
	refFactory->add(StockID["Mirror"], icon_set["Mirror"]);
	refFactory->add_default();

	Gtk::Stock::add(StockItem["RotateC"]);
	Gtk::Stock::add(StockItem["RotateCC"]);
	Gtk::Stock::add(StockItem["Mirror"]);

#if HAVE_GCONFMM	

	gconf_root = "/apps/gx";
	gconf_ui = "/apps/gx/ui";

	refClient = Conf::Client::get_default_client();

	try {
		// adding directories:
		refClient->add_dir(gconf_root, Conf::CLIENT_PRELOAD_NONE);
		refClient->add_dir(gconf_ui, Conf::CLIENT_PRELOAD_NONE);

		// setting notifies:
		refClient->notify_add(gconf_ui + "/toolbar",
				      sigc::mem_fun(*this, &Window::toolbar_key_changed));

		refClient->notify_add(gconf_ui + "/statusbar",
				      sigc::mem_fun(*this, &Window::statusbar_key_changed));

		// loading statusbar key:
		Conf::Entry entry = refClient->get_entry(gconf_ui + "/statusbar");
		if(entry.get_value().get_type() != Conf::VALUE_INVALID)
			old_statusbar_setting = refClient->get_bool(gconf_ui + "/statusbar");
		else {
			old_statusbar_setting = true; // default
			refClient->set(gconf_ui + "/statusbar", true);
		}
		
		// loading toolbar key:
		entry = refClient->get_entry(gconf_ui + "/toolbar");
		if(entry.get_value().get_type() != Conf::VALUE_INVALID)
			old_toolbar_setting = refClient->get_bool(gconf_ui + "/toolbar");
		else {
			old_toolbar_setting = true; // default
			refClient->set(gconf_ui + "/toolbar", true);
		}

		// loading window size:
		window_width_setting = refClient->get_int(gconf_ui + "/window_width");
		window_height_setting = refClient->get_int(gconf_ui + "/window_height");
	}
	catch(const Gnome::Conf::Error& e) {
		g_printerr("%s\n", e.what().data());
	}
#else

	old_statusbar_setting = true;
	old_toolbar_setting = true;
	
#endif // HAVE_GCONFMM

	if(not window_width_setting or not window_height_setting) { // invalid key(s) or 
		window_width_setting = win_width;                   // no gconfmm support enabled
		window_height_setting = win_height;
	}

	// Window settings:
	set_title(PACKAGE);
	set_position(Gtk::WIN_POS_CENTER);

	/// if HAVE_GCONFMM it should resize window to stored values, otherwise it uses
	/// default size (win_width, win_height) setted above
	set_default_size(window_width_setting, window_height_setting);

//	refAccel = get_accel_group();

	// Menubar:
	refActionGroup = Gtk::ActionGroup::create();
	
	/// Menu File:
	refActionGroup->add(Gtk::Action::create("MenuFile", "_File"));
	refActionGroup->add(Gtk::Action::create("Open", Gtk::Stock::OPEN, "_Open..."),
			    sigc::mem_fun(*this, &Window::on_open));
	refActionGroup->add(Gtk::Action::create("Save", Gtk::Stock::SAVE),
			    Gtk::AccelKey("<control>s"),
			    sigc::bind(sigc::mem_fun(*this, &Window::on_save), false));
	refActionGroup->add(Gtk::Action::create("SaveAs", Gtk::Stock::SAVE_AS, "Save _as..."),
			    Gtk::AccelKey("<control><shift>s"),
			    sigc::mem_fun(*this, &Window::on_save_as));

	refActionGroup->add(Gtk::Action::create("Close", Gtk::Stock::CLOSE, "_Close directory"),
			    sigc::mem_fun(*this, &Window::on_directory_close));
	refActionGroup->add(Gtk::Action::create("Quit", Gtk::Stock::QUIT),
			    sigc::mem_fun(*this, &Window::hide));
	
	/// Menu Edit:
	refActionGroup->add(Gtk::Action::create("MenuEdit", "_Edit"));
	refActionGroup->add(Gtk::Action::create("Undo", Gtk::Stock::UNDO),
			    Gtk::AccelKey("<control>z"),
			    sigc::mem_fun(*this, &Window::on_undo));
	refActionGroup->add(Gtk::Action::create("RotateC", StockID["RotateC"], "Rotate Clockwise"),
			    Gtk::AccelKey("<control>r"),
			    sigc::bind(sigc::mem_fun(*this, &Window::on_rotate),
				       90));
	refActionGroup->add(Gtk::Action::create("RotateCC", StockID["RotateCC"], "Rotate CounterClockwise"),
			    Gtk::AccelKey("<control><shift>r"),
			    sigc::bind(sigc::mem_fun(*this, &Window::on_rotate),
				       -90));
	refActionGroup->add(Gtk::Action::create("Mirror", StockID["Mirror"], "_Mirror"),
			    Gtk::AccelKey("<control>m"),
			    sigc::mem_fun(*this, &Window::on_mirror));
	refActionGroup->add(Gtk::Action::create("Saturate", "Saturate..."),
			    sigc::mem_fun(*this, &Window::on_saturate));
	refActionGroup->add(Gtk::Action::create("Scale", "Scale image..."),
			    sigc::mem_fun(*this, &Window::on_scale));
	
	refUIManager = Gtk::UIManager::create();
	refUIManager->insert_action_group(refActionGroup);

	/// Menu View:
	refActionGroup->add(Gtk::Action::create("MenuView", "_View"));
	refActionGroup->add(Gtk::ToggleAction::create("ToggleToolbar", "_Toolbar",
						      "Show/hide toolbar", old_toolbar_setting),
			    Gtk::AccelKey("<alt>t"),
			    sigc::mem_fun(*this, &Window::on_toggle_toolbar));
	refActionGroup->add(Gtk::ToggleAction::create("ToggleStatusbar", "_Statusbar",
						      "Show/hide statusbar", old_statusbar_setting),
			    Gtk::AccelKey("<alt>s"),
			    sigc::mem_fun(*this, &Window::on_toggle_statusbar));
	refActionGroup->add(Gtk::Action::create("ZoomFit", Gtk::Stock::ZOOM_FIT),
			    Gtk::AccelKey("space"),
			    sigc::mem_fun(*this, &Window::on_zoom_fit));
	refActionGroup->add(Gtk::Action::create("Zoom100", Gtk::Stock::ZOOM_100),
			    Gtk::AccelKey("1"),
			    sigc::mem_fun(*this, &Window::on_reset_zoom));

	refActionGroup->add(Gtk::Action::create("ZoomIn", Gtk::Stock::ZOOM_IN),
			    Gtk::AccelKey("plus"),
			    sigc::mem_fun(*this, &Window::on_zoom_in));
	refActionGroup->add(Gtk::Action::create("ZoomOut", Gtk::Stock::ZOOM_OUT),
			    Gtk::AccelKey("minus"),
			    sigc::mem_fun(*this, &Window::on_zoom_out));
	refActionGroup->add(Gtk::Action::create("Info", Gtk::Stock::INFO,
						"Image _info..."),
			    Gtk::AccelKey("<control>i"),
			    sigc::mem_fun(*this, &Window::on_show_info));

	/// Menu Go:
	refActionGroup->add(Gtk::Action::create("MenuGo", "_Go"));
	refActionGroup->add(Gtk::Action::create("Prev", Gtk::Stock::GO_BACK, "_Previous Image"),
			    Gtk::AccelKey("<alt>Left"),
			    sigc::mem_fun(*this, &Window::goto_prev));
	refActionGroup->add(Gtk::Action::create("Next", Gtk::Stock::GO_FORWARD, "_Next Image"),
			    Gtk::AccelKey("<alt>Right"),
			    sigc::mem_fun(*this, &Window::goto_next));
	refActionGroup->add(Gtk::Action::create("First", Gtk::Stock::GOTO_FIRST, "_First Image"),
			    Gtk::AccelKey("<alt>Home"),
			    sigc::mem_fun(*this, &Window::goto_first));
	refActionGroup->add(Gtk::Action::create("Last", Gtk::Stock::GOTO_LAST, "_Last Image"),
			    Gtk::AccelKey("<alt>End"),
			    sigc::mem_fun(*this, &Window::goto_last));

	/// Menu Help:
	refActionGroup->add(Gtk::Action::create("MenuHelp", "_Help"));
	refActionGroup->add(Gtk::Action::create("Supported", Gtk::Stock::DIALOG_INFO,
						"Supported _formats"),
			    sigc::mem_fun(*this, &Window::show_supported_files));
	refActionGroup->add(Gtk::Action::create("About", Gtk::Stock::ABOUT),
			    sigc::mem_fun(*this, &Window::show_about));

	// Image Popup Menu:
 	refActionGroup->add(Gtk::Action::create("Reload", Gtk::Stock::REFRESH, "_Reload image"),
 			    sigc::mem_fun(*this, &Window::on_image_reload));

	// Toolbar Popup Menu:
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleOpen", "Open", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleSave", "Save", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleSep", "Separator", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleZoomFit", "Best Fit", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleZoom100", "Normal Size", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleZoomIn", "Zoom In", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleZoomOut", "Zoom Out", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleRotateC", "Rotate Clockwise", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleRotateCC", "Rotate CounterClockwise", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleFirst", "Go to first", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("TogglePrev", "Go to prev", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleNext", "Go to next", "", true));
//  	refActionGroup->add(Gtk::ToggleAction::create("ToggleLast", "Go to last", "", true));

	Glib::ustring ui_info =
		"<ui>"
		"  <menubar name='MenuBar'>"
		"    <menu action='MenuFile'>"
	        "      <menuitem action='Open' />"
		"      <separator />"
		"      <menuitem action='Save' />"
		"      <menuitem action='SaveAs' />"
		"      <separator />"
		"      <menuitem action='Close' />"
		"      <menuitem action='Quit' />"
		"    </menu>"
	        "    <menu action='MenuEdit'>"
		"      <menuitem action='Undo' />"
		"      <separator />"
	        "      <menuitem action='RotateC' />"
	        "      <menuitem action='RotateCC' />"
		"      <menuitem action='Mirror' />"
		"      <separator />"
		"      <menuitem action='Saturate' />"
		"      <menuitem action='Scale' />"
                "      </menu>"
		"    <menu action='MenuView'>"
		"      <menuitem action='ToggleToolbar' />"
		"      <menuitem action='ToggleStatusbar' />"
		"      <separator />"
		"      <menuitem action='ZoomFit' />"
		"      <menuitem action='Zoom100' />"
		"      <menuitem action='ZoomIn' />"
		"      <menuitem action='ZoomOut' />"
		"      <separator />"
		"      <menuitem action='Info' />"
		"    </menu>"
		"    <menu action='MenuGo'>"
		"      <menuitem action='Prev' />"
		"      <menuitem action='Next' />"
		"      <separator />"		
		"      <menuitem action='First' />"
		"      <menuitem action='Last' />"
		"    </menu>"
		"    <menu action='MenuHelp'>"
		"      <menuitem action='Supported' />"
		"      <menuitem action='About' />"
		"    </menu>"
              	"  </menubar>"
		"  <toolbar name='Toolbar'>"
		"    <toolitem action='Open' />"
		"    <toolitem action='Save' />"
		"    <separator />"
		"    <toolitem action='ZoomFit' />"
		"    <toolitem action='Zoom100' />"
		"    <toolitem action='ZoomIn' />"
		"    <toolitem action='ZoomOut' />"
		"    <separator />"
		"    <toolitem action='RotateC' />"
		"    <toolitem action='RotateCC' />"
		"    <separator />"
		"    <toolitem action='First' />"
		"    <toolitem action='Prev' />"
		"    <toolitem action='Next' />"
		"    <toolitem action='Last' />"
		"  </toolbar>"
		"  <popup name='ImagePopup'>"
		"    <menuitem action='Prev' />"
		"    <menuitem action='Next' />"
		"    <separator/>"
		"    <menuitem action='RotateC' />"
		"    <menuitem action='RotateCC' />"
		"    <separator/>"
		"    <menuitem action='ZoomFit' />"
		"    <menuitem action='Zoom100' />"
		"    <menuitem action='ZoomIn' />"
		"    <menuitem action='ZoomOut' />"
		"    <separator/>"
		"    <menuitem action='Reload' />"
		"    <menuitem action='Close' />"
		"  </popup>"
//  		"  <popup name='ToolPopup'>"
//  		"    <menuitem action='ToggleOpen' />"
//  		"    <menuitem action='ToggleSave' />"
// 		"    <menuitem action='ToggleSep0' >"
//  		"    <menuitem action='ToggleZoomFit' />"
//  		"    <menuitem action='ToggleZoom100' />"
//  		"    <menuitem action='ToggleZoomIn' />"
//  		"    <menuitem action='ToggleZoomOut' />"
// 		"    <menuitem action='ToggleSep1' >"
//  		"    <menuitem action='ToggleRotateC' />"
//  		"    <menuitem action='ToggleRotateCC' />"
// 		"    <menuitem action='ToggleSep2' >"
//  		"    <menuitem action='ToggleFirst' />"
//  		"    <menuitem action='TogglePrev' />"
//  		"    <menuitem action='ToggleNext' />"
//  		"    <menuitem action='ToggleLast' />"
//  		"  </popup>"
		"</ui>";

	refUIManager->add_ui_from_string(ui_info);

	// Statusbar:
	status0.set_has_resize_grip(false);
	status1.set_has_resize_grip(false);

	status0.set_size_request(300);
	status0.set_size_request(300);

	status_box.pack_start(status0, true, true, 0);
	status_box.pack_start(status1);//, false, true, 0);
	status_box.pack_start(status2);//, false, true, 0);

	// Setting widgets:
	/// Menu:
	menu["Open"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuFile/Open"));
	menu["Save"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuFile/Save"));
	menu["SaveAs"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuFile/SaveAs"));
	menu["Close"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuFile/Close"));
	menu["Quit"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuFile/Quit"));

	menu["Undo"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuEdit/Undo"));

	menu["Mirror"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuEdit/Mirror"));
	menu["RotateC"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuEdit/RotateC"));
	menu["RotateCC"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuEdit/RotateCC"));
	menu["Saturate"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuEdit/Saturate"));
	menu["Scale"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuEdit/Scale"));

	menu["ZoomFit"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuView/ZoomFit"));
	menu["Zoom100"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuView/Zoom100"));
	menu["ZoomIn"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuView/ZoomIn"));
	menu["ZoomOut"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuView/ZoomOut"));
        menu["Info"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuView/Info"));

	menu["Prev"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuGo/Prev"));
	menu["Next"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuGo/Next"));
	menu["First"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuGo/First"));
	menu["Last"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuGo/Last"));

	menu["Supported"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuHelp/Supported"));
	menu["About"] = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuHelp/About"));

	/// Toolbar:
	toolbar["Open"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/Open"));
	toolbar["Save"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/Save"));

	toolbar["ZoomFit"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/ZoomFit"));
	toolbar["Zoom100"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/Zoom100"));
	toolbar["ZoomIn"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/ZoomIn"));
	toolbar["ZoomOut"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/ZoomOut"));

	toolbar["RotateC"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/RotateC"));
	toolbar["RotateCC"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/RotateCC"));

	toolbar["First"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/First"));
        toolbar["Prev"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/Prev"));
	toolbar["Next"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/Next"));
	toolbar["Last"] = static_cast<Gtk::ToolButton*>(refUIManager->get_widget("/Toolbar/Last"));

	/// Image Popup menu:
	image_popup["Prev"] = refUIManager->get_widget("/ImagePopup/Prev");
	image_popup["Next"] = refUIManager->get_widget("/ImagePopup/Next");

	image_popup["RotateC"] =  refUIManager->get_widget("/ImagePopup/RotateC");
	image_popup["RotateCC"] = refUIManager->get_widget("/ImagePopup/RotateCC");

	image_popup["ZoomFit"] = refUIManager->get_widget("/ImagePopup/ZoomFit");
	image_popup["Zoom100"] = refUIManager->get_widget("/ImagePopup/Zoom100");
	image_popup["ZoomIn"] = refUIManager->get_widget("/ImagePopup/ZoomIn");
	image_popup["ZoomOut"] = refUIManager->get_widget("/ImagePopup/ZoomOut");
	
	image_popup["Reload"] = refUIManager->get_widget("/ImagePopup/Reload");
	image_popup["Close"] = refUIManager->get_widget("/ImagePopup/Close");
	
	/// Actions:
	Glib::RefPtr<Gtk::Action> act = refUIManager->get_action("/MenuBar/MenuView/ToggleToolbar");
	
	toggle_action["Toolbar"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

	act = refUIManager->get_action("/MenuBar/MenuView/ToggleStatusbar");
	toggle_action["Statusbar"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleOpen");
// 	toggle_action["ToggleOpen"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleSave");
// 	toggle_action["ToggleSave"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleSep0");
// 	toggle_action["ToggleSep0"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleZoomFit");
// 	toggle_action["ToggleZoomFit"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleZoom100");
// 	toggle_action["ToggleZoom100"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleZoomIn");
// 	toggle_action["ToggleZoomIn"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleZoomOut");
// 	toggle_action["ToggleZoomOut"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleSep1");
// 	toggle_action["ToggleSep1"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleRotateC");
// 	toggle_action["ToggleRotateC"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleRotateCC");
// 	toggle_action["ToggleRotateCC"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleSep2");
// 	toggle_action["ToggleSep2"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleFirst");
// 	toggle_action["ToggleFirst"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/TogglePrev");
// 	toggle_action["TogglePrev"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleNext");
// 	toggle_action["ToggleNext"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

// 	act = refUIManager->get_action("/ToolPopup/ToggleLast");
// 	toggle_action["ToggleLast"] = Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(act);

	// Packing menubar:
	Gtk::Widget* pMenubar = refUIManager->get_widget("/MenuBar");
	main_vbox.pack_start(*pMenubar, false, false, 0);

	scroll_win.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	image_eventbox.add(image);
	scroll_win.add(image_eventbox);

	// Tooltips:
	toolbar["Open"]->set_tooltip(tooltips, "Open an image");
	toolbar["Save"]->set_tooltip(tooltips, "Save the current image");
	toolbar["ZoomFit"]->set_tooltip(tooltips, "Fit image to window");
	toolbar["Zoom100"]->set_tooltip(tooltips, "Normal zoom");
	toolbar["ZoomIn"]->set_tooltip(tooltips, "Zoom in");
	toolbar["ZoomOut"]->set_tooltip(tooltips, "Zoom out");
	toolbar["RotateC"]->set_tooltip(tooltips, "Rotate clockwise");
	toolbar["RotateCC"]->set_tooltip(tooltips, "Rotate counteclockwise");
	toolbar["First"]->set_tooltip(tooltips, "Go to first image");
	toolbar["Prev"]->set_tooltip(tooltips, "Go to the previous image");
	toolbar["Next"]->set_tooltip(tooltips, "Go to the next image");
	toolbar["Last"]->set_tooltip(tooltips, "Go to last image");

	// Accelerators:
	refAccel = (refUIManager->get_accel_group());
	add_accel_group(refAccel);

	// Signals:
	image_eventbox.signal_scroll_event().connect(sigc::mem_fun(*this,
							     &Window::on_image_scroll_event));

	image_eventbox.signal_button_press_event().connect(sigc::mem_fun(*this,
								   &Window::on_image_button_press_event));
//	toolbar_eventbox.signal_button_press_event().connect(sigc::mem_fun(*this,
//									   &Window::on_toolb_button_press_event));

	signal_check_resize().connect(sigc::mem_fun(*this,
						    &Window::on_window_resize));

	// Menu signals:
	/// Menu File:
	menu["Open"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Open an image"));
	menu["Open"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["Save"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Save the current image"));
	menu["Save"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["SaveAs"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Save the current image with a different name"));
	menu["SaveAs"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["Close"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Close all the images in the current directory"));
	menu["Close"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));
	
	menu["Quit"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Quit the program"));
	menu["Quit"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	/// Menu Edit:
	menu["Undo"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Undo the last action"));
	menu["Undo"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["Mirror"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Apply a mirror effect on the image"));
	menu["Mirror"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["RotateC"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Rotate 90 degrees clockwise"));
	menu["RotateC"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["RotateCC"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Rotate 90 degrees counterclockwise"));
	menu["RotateCC"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["Saturate"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Adjust the saturation level"));
	menu["Saturate"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["Scale"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Change the size of the image"));
	menu["Scale"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	/// Menu View:
	Gtk::MenuItem* widget = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuView/ToggleToolbar"));
	widget->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Show or hide the toolbar"));
	widget->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	widget = static_cast<Gtk::MenuItem*>(refUIManager->get_widget("/MenuBar/MenuView/ToggleStatusbar"));	
	widget->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Show or hide the statusbar"));
	widget->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["ZoomFit"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Fit image to window"));
	menu["ZoomFit"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["Zoom100"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Reset zoom to 100%"));
	menu["Zoom100"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["ZoomIn"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "The zoom factor will be increased by 20%"));
	menu["ZoomIn"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["ZoomOut"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "The zoom factor will be decreased by 20%"));
	menu["ZoomOut"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["Info"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Show information about the current image"));
	menu["Info"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	/// Menu Help:
	menu["Supported"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "Show a list of supported formats by gx"));
	menu["Supported"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	menu["About"]->signal_select().connect(sigc::bind(sigc::mem_fun(*this,
								    &Window::on_menuitem_selected),
						      "About this application"));
	menu["About"]->signal_deselect().connect(sigc::mem_fun(*this, &Window::on_menuitem_deselected));

	// End packing:
	vbox.pack_start(scroll_win);
	
	pToolbar = static_cast<Gtk::Toolbar*>(refUIManager->get_widget("/Toolbar"));
	pToolbar->set_toolbar_style(Gtk::TOOLBAR_ICONS);
	toolbar_eventbox.add(*pToolbar);
	vbox.pack_start(toolbar_eventbox, false, false, 0);

	paned.pack1(vbox);
	main_vbox.pack_start(paned);
	main_vbox.pack_start(status_box, false, false, 0);

	// Go!
	add(main_vbox);
	show_all_children();

	image.set_double_buffered(false);

	if(not old_toolbar_setting)
		pToolbar->hide();

	if(not old_statusbar_setting)
		status_box.hide();

	// Image management:
	if(files.empty())
		load_gx_image();
	else {
		std::string first_image = *files.begin();
//		files.sort();

		for(FileCIter i=files.begin(); i!=files.end(); i++) {
			try {
				if(Utils::is_directory(*i))
					set_directory(*i);
				else {
					if(files.size() == 1)
						if(not Utils::is_supported(*i))
							throw Error::Unsupported(*i);

					set_directory(Glib::path_get_dirname(*i));
				}
			}
			catch(const Error::Unsupported& e) {
				error_message("Unsupported format",
					      Utils::get_mime_type(e.get_path()));
				exit(0);
			}
			catch(const Error::Access& e) {
				error_message("File not found",
						      e.get_path());
				exit(0);
			}
		}
		if(Utils::is_directory(first_image))
			set_image_from_iter(img_list.begin());
		else
			set_image_from_iter(get_image_iter(first_image));
	}

	set_icon(Gdk::Pixbuf::create_from_inline(-1, icon, false));
}

Window::~Window()
{

#if HAVE_GCONFMM
	
	int x, y;
	get_size(x, y);

	if(x != window_width_setting)
		refClient->set(gconf_ui + "/window_width", x);

	if(y != window_height_setting)
		refClient->set(gconf_ui + "/window_height", y);

#endif // HAVE_GCONFMM

}

// Open an image calling a Gtk::FileChooserDialog:
void Window::on_open()
{
	Gtk::FileChooserDialog chooser("Open an image",
				       Gtk::FILE_CHOOSER_ACTION_OPEN);
	chooser.set_transient_for(*this);
	
	chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	chooser.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	// Set the filters:
	Gtk::FileFilter filter_images; // images
	filter_images.set_name("Images");

	UstringList l = Utils::get_supported_types();
	for(UstringCIter i=l.begin(); i!=l.end(); i++)
		filter_images.add_mime_type(*i);

	Gtk::FileFilter filter_any; // any files
	filter_any.set_name("Any Files");
	filter_any.add_pattern("*");

	// Add the filters to chooser dialog:
	chooser.add_filter(filter_images);
	chooser.add_filter(filter_any);

	int res = chooser.run();
	chooser.hide();

	if(res == Gtk::RESPONSE_OK) {
		std::string image_path = chooser.get_filename();

		if(not Utils::is_supported(image_path))
			error_message("Unsupported format",
				      Utils::get_mime_type(image_path));
		else {
			set_directory(Glib::path_get_dirname(image_path));
			set_image_from_iter(get_image_iter(image_path));
		}
	}
}

void Window::on_save(bool save_as)
{
	if(not toolbar["Save"]->is_sensitive() and not save_as)
		return;

	try {
		image.save(save_as);
	}
	catch(const Image::Error::Unwritable& err) {
		error_message("Unwritable file", Utils::get_mime_type(image.get_path()));
		return;
	}
	catch(const Glib::FileError& err) {
		Glib::ustring error = Utils::parse_io_error(err.code());

		error_message(error, image.get_path());
		return;
	}
	catch(const Gdk::PixbufError& err) {
		Glib::ustring error = Utils::parse_pixbuf_error(err.code());

		error_message(error, image.get_path());
		return;
	}
	catch(...) {
		error_message("Unknown error", image.get_path());
		return;
	}
		
	if(not save_as)
		set_save_buttons_sensitive(false);

	if(status_box.is_visible())
		update_statusbars(image.get_name() + " saved", true);
}

void Window::on_save_as()
{
	if(not menu["SaveAs"]->is_sensitive())
		return;

	Gtk::FileChooserDialog dialog(*this, "Save as...",
				      Gtk::FILE_CHOOSER_ACTION_SAVE);
	
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);

	int res = dialog.run();

	if(res == Gtk::RESPONSE_OK) {
		std::string new_path = dialog.get_filename();
		std::string old_path = image.get_path();

		image.set_path(new_path);
		on_save(true); 
		image.set_path(old_path);

		// I know, it's a trick.. but works :)
	}
}

// Show about dialog:
void Window::show_about() const
{
	using namespace Gx::Info;

	Gtk::AboutDialog dialog;

	UstringList authors = load_authors();
	UstringList artists = load_artists();

	dialog.set_authors(authors);
	dialog.set_artists(artists);
	dialog.set_comments(comment);
	dialog.set_copyright(COPYRIGHT);
	dialog.set_name(PACKAGE);
	dialog.set_version(VERSION);
	dialog.set_website(WWW);
	dialog.set_license(license);
	dialog.set_logo(Gdk::Pixbuf::create_from_inline(-1, icon, false));

	dialog.run();
}

ImageIter Window::get_image_iter(const std::string& path)
{
	if(not img_list.size())
		throw "WTF?!?";

	ImageIter i = img_list.begin();

	while( (*i).get_path() != path and i != img_list.end() )
	      i++;

	return (i != img_list.end()) ? i : img_iter;
}

// Load an image from path:
void Window::set_image_from_path(const std::string& path)
{
	image = Gx::Image(path);

	img_list.push_back(image);
	img_iter = --img_list.end();
}

// Load an image from iter:
void Window::set_image_from_iter(ImageIter i, bool ignore_previous)
{
	if(first_session) first_session = false;

	if(img_list.empty() or i == img_list.end()) {
		load_gx_image();
		return;
	}

	// note: ignore_previous is true when img_iter has been deleted!
	if(not ignore_previous and i != img_iter)
		*img_iter = image; // store updated image

	img_iter = i; // change iter

	// load the image number:
	unsigned count = 1;
	for(ImageIter j=img_list.begin(); j!=img_list.end(); j++, count++)
		if(j == i)
			break;

	img_count = count;

	// now we can draw the image:
	image = *img_iter;
	image.load();

	if(image.is_fitted()) {
		image.zoom_fit(s_win_x, s_win_y, true); 
		if(image.get_actual_zoom() < 0)
			g_printerr("Ignore above message ---^\n"); // it works... 
	}

	if(not image_popup["Close"]->is_sensitive())
		image_popup["Close"]->set_sensitive();

	if(not menu["Close"]->is_sensitive())
		menu["Close"]->set_sensitive();

	if(not image_popup["Reload"]->is_sensitive())
		image_popup["Reload"]->set_sensitive();

	general_update();
	update_statusbars(image.get_name());
}

// Load a directory:
void Window::set_directory(const std::string& path)
{
	FileList l;
	ImageIter tmp_iter;

	close_directory(path);

	if(img_list.empty())
		tmp_iter = img_list.begin();
	else
		tmp_iter = --img_list.end();

	try {
		l = Utils::visit_directory(path);
	}
	catch(const Error::Dir&) {
		g_printerr("gx: Error opening %s\n", path.c_str());
		return;
	}

	if(l.empty() and img_list.empty())
		load_gx_image();

	if(not l.empty())
		for(FileCIter i=l.begin(); i!=l.end(); i++)
			set_image_from_path(*i);
}

void Window::load_gx_image()
{
	image.set_from_pixbuf(Gdk::Pixbuf::create_from_inline(-1, logo, false));

	image_popup["Reload"]->set_sensitive(false);
	image_popup["Close"]->set_sensitive(false);
	set_edit_buttons_sensitive(false);
	set_save_buttons_sensitive(false);
	menu["SaveAs"]->set_sensitive(false);
	menu["Close"]->set_sensitive(false);
	menu["Undo"]->set_sensitive(false);
	menu["Info"]->set_sensitive(false);

	update_navigation_buttons();
	reset_statusbars();

	if(first_session) {
		update_statusbars("Welcome to gx!", true);
		first_session = false;
	}
}

// Show the popup image menu:
bool Window::on_image_button_press_event(const GdkEventButton* event) const
{
	if(event->type == GDK_BUTTON_PRESS and (event->button == 3)) {
		Gtk::Menu* pMenu = static_cast<Gtk::Menu*>(refUIManager->get_widget("/ImagePopup"));

		if(pMenu)
			pMenu->popup(event->button, event->time);
		
		return true;
	}
	else
		return false;
}

// bool Window::on_toolb_button_press_event(GdkEventButton* event)
// {
// 	if(event->type == GDK_BUTTON_PRESS and (event->button == 3)) {
// 		Gtk::Menu* pMenu = 
// 			static_cast<Gtk::Menu*>(refUIManager->get_widget("/ToolPopup"));

// 		if(pMenu)
// 			pMenu->popup(event->button, event->time);
		
// 		return true;
// 	}
// 	else
// 		return false;
// }

// Handle the scroll events:
bool Window::on_image_scroll_event(const GdkEventScroll* event)
{
	if(not image.is_editable())
		return false;

	bool flag = false;

	int actual_zoom = image.get_actual_zoom();

	if(event->direction == GDK_SCROLL_UP) {
		if(toolbar["ZoomIn"]->sensitive()) {
//			if(actual_zoom + 3 > image.get_max_zoom())
				image.set_zoom(image.get_max_zoom());
//			else
				image.set_zoom(actual_zoom + 3);

			flag = true;
		}
	}
	else if(event->direction == GDK_SCROLL_DOWN) {
		if(toolbar["ZoomOut"]->sensitive()) {
			image.set_zoom(actual_zoom - 3);
			flag = true;
		}
	}

	if(flag) {
		update_zoom_buttons();
		update_statusbars();
	}

	return flag;
}

void Window::on_window_resize()
{
	if(img_list.empty())
		return;

	update_scrolledwindow_size();

	if(image.is_fitted() 
	   and
	   image.get_fit_zoom_value(s_win_x, s_win_y) != image.get_actual_zoom())
		on_zoom_fit();
}

// Handle the toolbar's buttons:
void Window::set_edit_buttons_sensitive(bool flag)
{
	// Toolbar
	toolbar["ZoomFit"]->set_sensitive(flag);
	toolbar["Zoom100"]->set_sensitive(flag);
	toolbar["ZoomIn"]->set_sensitive(flag);
	toolbar["ZoomOut"]->set_sensitive(flag);
	toolbar["RotateC"]->set_sensitive(flag);
	toolbar["RotateCC"]->set_sensitive(flag);

	// Menu
	menu["Mirror"]->set_sensitive(flag);
	menu["ZoomFit"]->set_sensitive(flag);
	menu["Zoom100"]->set_sensitive(flag);
	menu["ZoomIn"]->set_sensitive(flag);
	menu["ZoomOut"]->set_sensitive(flag);
	menu["RotateC"]->set_sensitive(flag);
	menu["RotateCC"]->set_sensitive(flag);
	menu["Saturate"]->set_sensitive(flag);
	menu["Scale"]->set_sensitive(flag);

	// Popup menu
	image_popup["ZoomFit"]->set_sensitive(flag);
	image_popup["Zoom100"]->set_sensitive(flag);
	image_popup["ZoomIn"]->set_sensitive(flag);
	image_popup["ZoomOut"]->set_sensitive(flag);
	image_popup["RotateC"]->set_sensitive(flag);	
	image_popup["RotateCC"]->set_sensitive(flag);
}

// Handle the save buttons (menu and toolbar)
void Window::set_save_buttons_sensitive(bool flag)
{
	toolbar["Save"]->set_sensitive(flag);
	menu["Save"]->set_sensitive(flag);
}

void Window::update_misc_buttons()
{
	if(img_list.empty()) {
		set_save_buttons_sensitive(false);
		menu["Undo"]->set_sensitive(false);
		return;
	}

	if(image.is_invalid()) {
		menu["Undo"]->set_sensitive(false);
		menu["SaveAs"]->set_sensitive(false);
		menu["Info"]->set_sensitive(false);
		set_save_buttons_sensitive(false);
	}
	else {
		menu["Info"]->set_sensitive();
		menu["SaveAs"]->set_sensitive();
	}

	if(image.is_modified() and image.is_writable())
		set_save_buttons_sensitive();
	else
		set_save_buttons_sensitive(false);

	menu["Undo"]->set_sensitive(image.has_undo());

	if(image.is_animated()) {
		set_edit_buttons_sensitive(false);
		menu["SaveAs"]->set_sensitive(false);
	}

	set_edit_buttons_sensitive(image.is_editable());
}

void Window::update_navigation_buttons()
{
	if(img_list.size() <= 1) {
		menu["First"]->set_sensitive(false);
		menu["Prev"]->set_sensitive(false);
		menu["Next"]->set_sensitive(false);
		menu["Last"]->set_sensitive(false);

		toolbar["First"]->set_sensitive(false);
		toolbar["Prev"]->set_sensitive(false);
		toolbar["Next"]->set_sensitive(false);
		toolbar["Last"]->set_sensitive(false);

		image_popup["Prev"]->set_sensitive(false);
		image_popup["Next"]->set_sensitive(false);
	}
 	else {
 		if(img_iter == --img_list.end()) {
			menu["First"]->set_sensitive();
			menu["Prev"]->set_sensitive();
 			menu["Next"]->set_sensitive(false);
 			menu["Last"]->set_sensitive(false);

			toolbar["First"]->set_sensitive();
			toolbar["Prev"]->set_sensitive();
 			toolbar["Next"]->set_sensitive(false);
 			toolbar["Last"]->set_sensitive(false);

			image_popup["Prev"]->set_sensitive();
 			image_popup["Next"]->set_sensitive(false);
 		}
		else if(img_iter == img_list.begin()) {
			menu["First"]->set_sensitive(false);
 			menu["Prev"]->set_sensitive(false);
			menu["Next"]->set_sensitive();
			menu["Last"]->set_sensitive();

			toolbar["First"]->set_sensitive(false);
 			toolbar["Prev"]->set_sensitive(false);
			toolbar["Next"]->set_sensitive();
			toolbar["Last"]->set_sensitive();

			image_popup["Next"]->set_sensitive();
 			image_popup["Prev"]->set_sensitive(false);
 		}
		else {
			menu["First"]->set_sensitive();
			menu["Prev"]->set_sensitive();
			menu["Next"]->set_sensitive();
			menu["Last"]->set_sensitive();

			toolbar["First"]->set_sensitive();
			toolbar["Prev"]->set_sensitive();
			toolbar["Next"]->set_sensitive();
			toolbar["Last"]->set_sensitive();
			
			image_popup["Prev"]->set_sensitive();
			image_popup["Next"]->set_sensitive();
		}
 	}
}

void Window::update_statusbars(const std::string& msg, bool timeout)
{
	// statusbar 0
	if(msg != "") {
		if(not timeout)
			status0.pop();

		status0.push(msg, timeout);

		if(timeout)
			timeout_connection = Glib::signal_timeout().connect(
				sigc::mem_fun(*this, &Window::timeout_callback), 
				2000);
	}

	// statusbar 1:
	if(image.is_animated()) {
		status1.pop();
		status1.push("100%");
	}
	else {
		char* buf = new char[6];
	
		g_snprintf(buf, 6, "%d%%", image.get_actual_zoom());
		status1.pop();
		status1.push(buf);

		delete[] buf;
	}

	if(not img_list.size())
		return;

	char* buf = new char[16];

	g_snprintf(buf, 16, "%d / %d", img_count, img_list.size());
	status2.pop();
	status2.push(buf);

	delete[] buf;
}

// Update the titlebar with the name of the image:
void Window::update_title()
{
	set_title(Glib::ustring("gx") + 
		  Glib::ustring(" - ") +
		  image.get_name());
}

void Window::update_zoom_buttons()
{
	bool flag = not image.is_animated() and not no_more_zoom_in();

	menu["ZoomIn"]->set_sensitive(flag);
	toolbar["ZoomIn"]->set_sensitive(flag);
	image_popup["ZoomIn"]->set_sensitive(flag);
			
	flag = not image.is_animated() and not no_more_zoom_out();

	menu["ZoomOut"]->set_sensitive(flag);
	toolbar["ZoomOut"]->set_sensitive(flag);
	image_popup["ZoomOut"]->set_sensitive(flag);
}

void Window::goto_first()
{
	if(not toolbar["First"]->is_sensitive())
		return;

	image.cleanup();
	*img_iter = image;
	set_image_from_iter(img_list.begin());
}

void Window::goto_prev()
{
	if(not toolbar["Prev"]->is_sensitive())
		return;

	image.cleanup();
	*img_iter = image;
	set_image_from_iter(--img_iter);
}

void Window::goto_next()
{
	if(not toolbar["Next"]->is_sensitive())
		return;

	if(img_iter == --img_list.end() or img_list.empty())
		return;

	image.cleanup();
	*img_iter = image;
	set_image_from_iter(++img_iter);
}

void Window::goto_last()
{
	if(not toolbar["Last"]->is_sensitive())
		return;

	if(img_iter == --img_list.end() or img_list.empty())
		return;

	image.cleanup();
	*img_iter = image;
	set_image_from_iter(--img_list.end());
}

void Window::on_saturate()
{
	Dialog::Saturate dialog(image);
	int res = dialog.run();

	if(res == Gtk::RESPONSE_APPLY) {
		image.saturate(dialog.get_value());
		update_misc_buttons();
	}
}

void Window::on_scale()
{
	Dialog::Scale dialog(image);
	int res = dialog.run();

	if(res == Gtk::RESPONSE_OK) {
		if(image.is_fitted())
			on_zoom_fit();

		update_misc_buttons();
	}
}

void Window::on_zoom_fit()
{
	if(not toolbar["ZoomFit"]->is_sensitive())
		return;

	image.zoom_fit(s_win_x, s_win_y);

	update_zoom_buttons();

	if(status_box.is_visible())
		update_statusbars();
}

void Window::on_reset_zoom()
{
	if(not toolbar["Zoom100"]->is_sensitive())
		return;

	image.reset_zoom();

	update_zoom_buttons();

	if(status_box.is_visible())
		update_statusbars();
}

void Window::on_zoom_in()
{
	if(not toolbar["ZoomIn"]->is_sensitive())
		return;

	image.make_zoom_in();

	update_zoom_buttons();

	if(status_box.is_visible())
		update_statusbars();
}

void Window::on_zoom_out()
{
	if(not toolbar["ZoomOut"]->is_sensitive())
		return;

	image.make_zoom_out();

	update_zoom_buttons();

	if(status_box.is_visible())
		update_statusbars();
}

void Window::on_mirror()
{
	image.mirror();

	update_misc_buttons();

	if(image.is_writable())
		set_save_buttons_sensitive();
}

void Window::on_rotate(int angle)
{
	if(not toolbar["RotateC"]->is_sensitive())
		return;

	try {
		image.rotate(angle);
	}
	catch(const Image::Error::Rotate& e) {
		g_printerr("gx error: rotation value (%d) error in "
			   "Gx::Window::on_rotate()\n"
			   "Please, report this message to %s\n\n", 
			   e.get_value(), MAIL);
	}

	if(image.is_fitted())
		on_zoom_fit();

	update_misc_buttons();

	if(image.is_writable())
		set_save_buttons_sensitive();
}

void Window::on_undo()
{
	if(not image.has_undo())
		return;

	image.undo();

	if(image.is_fitted()) {
		update_scrolledwindow_size();
		image.zoom_fit(s_win_x, s_win_y, true);
	}

	update_misc_buttons();
}

void Window::on_show_info()
{
	if(not menu["Info"]->is_sensitive())
		return;

	Dialog::Info dialog(image);

	dialog.run();
}

void Window::on_image_reload()
{
	image.restore();

	on_zoom_fit();

	update_misc_buttons();
}

void Window::on_directory_close()
{
	if(img_list.empty())
		return;

	close_directory(image.get_dirname());
}
void Window::close_directory(const std::string& dir)
{
	if(img_list.empty())
		return;

	ImageList l;
	std::string tmp;

	bool found_prev = false;

	for(ImageIter i=img_list.begin(); i!=img_list.end(); i++)
		if((*i).get_dirname() != dir)
			l.push_back(*i);
		else if(not found_prev and not l.empty()) {
			tmp = (*--l.end()).get_path();
			found_prev = true;
		}

	img_list.clear();
	img_list = l;

	if(not found_prev and not img_list.empty())
		tmp = (*img_list.begin()).get_path();

	img_list.empty() ? load_gx_image() : set_image_from_iter(get_image_iter(tmp), true);
}

bool Window::timeout_callback()
{
	status0.pop(1);
	timeout_connection.disconnect();
	return true;
}

void Window::on_toggle_statusbar()
{
	bool is_set = toggle_action["Statusbar"]->get_active();

#if HAVE_GCONFMM

	if(is_set != old_statusbar_setting) {
		old_statusbar_setting = is_set;
		refClient->set(gconf_ui + "/statusbar", is_set);
#endif
		if(is_set) {
			status_box.show();
			img_list.empty() ? reset_statusbars() : update_statusbars();
		}
		else
			status_box.hide();
#if HAVE_GCONFMM
	}
#endif

}

void Window::on_toggle_toolbar()
{
	bool is_set = toggle_action["Toolbar"]->get_active();

#if HAVE_GCONFMM

	if(is_set != old_toolbar_setting) {
		old_toolbar_setting = is_set;
		refClient->set(gconf_ui + "/toolbar", is_set);
#endif 
		is_set ? pToolbar->show() : pToolbar->hide();
#if HAVE_GCONFMM
	}
#endif
}

#if HAVE_GCONFMM

void Window::toolbar_key_changed(guint c_id, Conf::Entry entry)
{
	if(entry.get_value().get_type() == Conf::VALUE_BOOL) {
		bool new_toolbar_setting = entry.get_value().get_bool();
		
		if(new_toolbar_setting != old_toolbar_setting) {
			old_toolbar_setting = new_toolbar_setting;
			toggle_action["Toolbar"]->set_active(new_toolbar_setting);
		}
	}
}

void Window::statusbar_key_changed(guint c_id, Conf::Entry entry)
{
	if(entry.get_value().get_type() == Conf::VALUE_BOOL) {
		bool new_statusbar_setting = entry.get_value().get_bool();

		if(new_statusbar_setting != old_statusbar_setting) {
			old_statusbar_setting = new_statusbar_setting;
			toggle_action["Statusbar"]->set_active(new_statusbar_setting);
		}
	}
}

#endif // HAVE_GCONFMM
