// -*- C++ -*-
//
// window.h
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef GX_WINDOW
#define GX_WINDOW

#include "image.h"
#include "dialogs.h"
#include "utils.h"
#include <gtkmm/actiongroup.h>
#include <gtkmm/box.h>
#include <gtkmm/eventbox.h>
#include <gtkmm/menuitem.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/paned.h>
#include <gtkmm/progressbar.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/stockid.h>
#include <gtkmm/toggleaction.h>
#include <gtkmm/toolbar.h>
#include <gtkmm/toolbutton.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/window.h>
#include <map>

#include "icons.h"

#if HAVE_GCONFMM
# include <gconfmm/client.h>
#endif

namespace Gx {

	typedef std::list<Gx::Image>   ImageList;
	typedef ImageList::iterator    ImageIter;

	typedef std::map<const char*, Gtk::MenuItem*>     MenuItemMap;
	typedef std::map<const char*, Gtk::ToolButton*> ToolButtonMap;
	typedef std::map<const char*, Gtk::Widget*>         WidgetMap;

	typedef std::map<const char*, Glib::RefPtr<Gtk::ToggleAction> > RefToggleActionMap;

	typedef std::map<const char*, Gtk::StockID>     StockIDMap;
	typedef std::map<const char*, Gtk::StockItem> StockItemMap;
	typedef std::map<const char*, Gtk::IconSet>     IconSetMap;

	class Window : public Gtk::Window {
	public:
		Window(FileList& images);
		virtual ~Window();

	protected:
		virtual void           on_open  ();
		virtual void           on_save  (bool save_as =false);
		virtual void        on_save_as  ();
		virtual void     error_message  (const Glib::ustring&   primary_text,
						 const Glib::ustring& secondary_text); // inline

		virtual void show_supported_files () const; // inline
		virtual void        show_about    () const;

		virtual ImageIter  get_image_iter  (const std::string& path);

		virtual void  set_image_from_path  (const std::string&   path);
		virtual void  set_image_from_iter  (ImageIter  iter, 
						    bool ignore_previous =false);
		virtual void        set_directory  (const std::string&   path);

		virtual void     load_gx_image  ();

		virtual bool  on_image_button_press_event  (const GdkEventButton*) const;
//		virtual bool  on_toolb_button_press_event  (GdkEventButton*);
		virtual bool        on_image_scroll_event  (const GdkEventScroll*);
		virtual void             on_window_resize  ();

		virtual void   set_edit_buttons_sensitive  (bool =true);
		virtual void   set_save_buttons_sensitive  (bool =true);

		virtual void               general_update  (); // inline
		virtual void          update_misc_buttons  ();
		virtual void    update_navigation_buttons  ();
		virtual void            update_statusbars  (const std::string& msg ="",
							    bool timeout =false);
		virtual void             reset_statusbars  (); // inline
		virtual void                 update_title  ();
		virtual void          update_zoom_buttons  ();
		virtual void   update_scrolledwindow_size  (); // inline
		virtual void         on_menuitem_selected  (const std::string& tooltip); // inline
		virtual void       on_menuitem_deselected  (); // inline

		virtual void           goto_first   ();
		virtual void            goto_prev   ();
		virtual void            goto_next   ();
		virtual void            goto_last   ();
		virtual void          on_saturate   ();
		virtual void             on_scale   ();
		virtual void          on_zoom_fit   ();
		virtual void        on_reset_zoom   ();
		virtual void           on_zoom_in   ();
		virtual void          on_zoom_out   ();
		virtual void            on_mirror   ();
		virtual void            on_rotate   (int   angle);
		virtual void              on_undo   ();
		virtual void         on_show_info   ();
		virtual void      on_image_reload   ();
		virtual void   on_directory_close   ();
		virtual void      close_directory   (const std::string& dir);

		virtual bool      no_more_zoom_in   () const; // inline
		virtual bool     no_more_zoom_out   () const; // inline

		virtual bool      timeout_callback   ();

		virtual void   on_toggle_statusbar   ();
		virtual void     on_toggle_toolbar   ();

	private:
		Gtk::HPaned                 paned;
		Gtk::ScrolledWindow    scroll_win;
		Gx::ImageList            img_list;
		Gx::ImageIter            img_iter;
		unsigned                img_count;
		Gx::Image                   image;
		Gtk::EventBox      image_eventbox;
		Gtk::EventBox    toolbar_eventbox;
		bool                first_session;
		FileList       directories_opened;
		FileList            images_opened;

		Glib::RefPtr<Gtk::AccelGroup> refAccel;
		
		Gtk::VBox      main_vbox;
		Gtk::VBox           vbox;
		Gtk::Tooltips   tooltips;
		
		Glib::RefPtr<Gtk::ActionGroup> refActionGroup;
		Glib::RefPtr<Gtk::UIManager>     refUIManager;

		// Widgets:
		MenuItemMap                     menu;
		ToolButtonMap                toolbar;
		WidgetMap                image_popup;
		RefToggleActionMap     toggle_action;
		Gtk::Toolbar*               pToolbar;

		// Statusbars stuff
		Gtk::Statusbar         status0;
		Gtk::Statusbar         status1;
		Gtk::Statusbar         status2;
		Gtk::HBox           status_box;

		// Stocks stuff:
		IconSetMap icon_set;
		StockIDMap               StockID;
		StockItemMap           StockItem;

#if HAVE_GCONFMM

		void toolbar_key_changed(guint c_id, Gnome::Conf::Entry entry);
		void statusbar_key_changed(guint c_id, Gnome::Conf::Entry entry);

		// gconf client:
		Glib::RefPtr<Gnome::Conf::Client> refClient;

		Glib::ustring gconf_root;
		Glib::ustring gconf_ui;

#endif

		// Default window's size:
		static const gint win_width = 600;
		static const gint win_height = 600;

		guint window_width_setting;
		guint window_height_setting;

		// scrolled windows size: (updated dynamically)
		int s_win_x;
		int s_win_y;

		// statusbar/toolbar visible?
		bool old_statusbar_setting;
		bool old_toolbar_setting;

		sigc::connection timeout_connection;
	};

	inline void Window::error_message(const Glib::ustring& s,
					  const Glib::ustring& t)

	{
		Gtk::MessageDialog dialog(*this,
					  s,
					  false,
					  Gtk::MESSAGE_ERROR,
					  Gtk::BUTTONS_CLOSE,                                   
					  true);
		dialog.set_secondary_text(t);
		dialog.run();
	}

	inline void Window::show_supported_files() const
	{
		Dialog::Supported dialog;
		dialog.run();
	}

	inline void Window::general_update()
	{
		update_zoom_buttons();
		update_navigation_buttons();
		update_misc_buttons();
		update_title();
	}

	inline void Window::reset_statusbars()
	{
		status0.pop();
		status1.pop();
		status2.pop();
	}

	inline void Window::on_menuitem_selected(const std::string& tooltip)
	{
		status0.push(tooltip, 1);
	}

	inline void Window::on_menuitem_deselected()
	{
		status0.pop(1);
	}

	inline void Window::update_scrolledwindow_size()
	{
		s_win_x = static_cast<int>(scroll_win.get_hadjustment()->get_page_size());
		s_win_y = static_cast<int>(scroll_win.get_vadjustment()->get_page_size());
	}

	inline bool Window::no_more_zoom_in() const
	{
		if(image.get_actual_zoom() + 
		   image.get_zoom_factor() >= 
		   image.get_max_zoom())
			return true;
		else
			return false;
	}

	inline bool Window::no_more_zoom_out() const
	{
		int x = image.get_width() * image.get_actual_zoom() / 100;
		int y = image.get_width() * image.get_actual_zoom() / 100;
		
		if(x * (image.get_actual_zoom() - image.get_zoom_factor()) / 100 < 2 or
		   y * (image.get_actual_zoom() - image.get_zoom_factor()) / 100 < 2)
			return true;
		else
			return false;
	}

} // Gx

#endif // GX_WINDOW
